package com.fireworksdk.bridge.components.videofeed

import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import android.widget.RelativeLayout
import androidx.constraintlayout.widget.ConstraintLayout
import com.loopnow.fireworklibrary.FeedType
import com.loopnow.fireworklibrary.models.FeedLayout
import com.loopnow.fireworklibrary.views.VideoFeedView
import com.fireworksdk.bridge.R
import com.fireworksdk.bridge.models.*
import com.fireworksdk.bridge.utils.FWVideoPlayerUtils
import com.loopnow.fireworklibrary.views.PlaylistGroupFeedView


class FWVideoFeed(
  context: Context,
  attrs: AttributeSet?
) : RelativeLayout(context, attrs) {

  private var videoFeedViewContainer: ConstraintLayout? = null
  var videoFeedView: VideoFeedView? = null

  private var playlistGroupFeedViewContainer: ConstraintLayout? = null
  private var playlistGroupFeedView: PlaylistGroupFeedView? = null

  private val videoFeedPropsModel: FWVideoFeedPropsModel by lazy { FWVideoFeedPropsModel() }

  constructor(context: Context) : this(context, null)

  fun setProps(props: FWVideoFeedPropsModel?) {
    if (props != null) {
      videoFeedPropsModel.source = props.source
      videoFeedPropsModel.channel = props.channel
      videoFeedPropsModel.playlist = props.playlist
      videoFeedPropsModel.playlistGroup = props.playlistGroup
      videoFeedPropsModel.mode = props.mode
      videoFeedPropsModel.videoFeedConfiguration = props.videoFeedConfiguration
      videoFeedPropsModel.videoPlayerConfiguration = props.videoPlayerConfiguration
    }
    setData()
  }

  fun setSourceProps(source: String?) {
    videoFeedPropsModel.source = source
    setData()
  }

  fun setChannelProps(channel: String?) {
    videoFeedPropsModel.channel = channel
    setData()
  }

  fun setPlaylistProps(playlist: String?) {
    videoFeedPropsModel.playlist = playlist
    setData()
  }

  fun setPlaylistGroupProps(playlistGroup: String?) {
    videoFeedPropsModel.playlistGroup = playlistGroup
    setData()
  }

  fun setModeProps(mode: String?) {
    videoFeedPropsModel.mode = mode
    setData()
  }

  fun setVideoFeedConfigProps(config: FWVideoFeedConfigModel?) {
    videoFeedPropsModel.videoFeedConfiguration = config
    setData()
  }

  fun setVideoPlayerConfigProps(config: FWVideoPlayerConfigModel?) {
    videoFeedPropsModel.videoPlayerConfiguration = config
    setData()
  }

  fun refresh() {
    videoFeedView?.refreshFeed()
    // todo://
    // playlistGroupFeedView.refresh()
  }

  private fun setData() {
    val source = videoFeedPropsModel.source
    val mode = videoFeedPropsModel.mode
    val channel = videoFeedPropsModel.channel
    val playlist = videoFeedPropsModel.playlist
    val playlistGroup = videoFeedPropsModel.playlistGroup
    val videoFeedConfiguration = videoFeedPropsModel.videoFeedConfiguration
    val videoPlayerConfiguration = videoFeedPropsModel.videoPlayerConfiguration

    if (source.isNullOrBlank()) {
      return
    }
    when (source) {
      FWVideoFeedSource.PlaylistGroup.rawValue -> {
        setPlaylistGroupFeed(mode, playlistGroup, videoFeedConfiguration, videoPlayerConfiguration)
      }
      FWVideoFeedSource.Playlist.rawValue -> {
        setFeed(mode, channel, playlist, FeedType.PLAYLIST, videoFeedConfiguration, videoPlayerConfiguration)
      }
      FWVideoFeedSource.Channel.rawValue -> {
        setFeed(mode, channel, null, FeedType.CHANNEL, videoFeedConfiguration, videoPlayerConfiguration)
      }
      else -> {
        setFeed(mode, null, null, FeedType.DISCOVER, videoFeedConfiguration, videoPlayerConfiguration)
      }
    }
  }

  private fun setFeed(
    mode: String?,
    channelId: String?, playlistId: String?, feedType: FeedType?,
    videoFeedConfiguration: FWVideoFeedConfigModel?,
    videoPlayerConfiguration: FWVideoPlayerConfigModel?,
  ) {
    // add view
    addVideoFeedView()

    // set layout
    val feedLayout = when {
      mode.equals(FWVideoFeedMode.Column.rawValue) -> FeedLayout.VERTICAL
      mode.equals(FWVideoFeedMode.Grid.rawValue) -> FeedLayout.GRID
      else -> FeedLayout.HORIZONTAL
    }
    videoFeedView?.setLayout(feedLayout)

    // set channel playlistId feedType
    if (feedType == FeedType.CHANNEL && !channelId.isNullOrBlank()) {
      videoFeedView?.setFeed(channelId, null, FeedType.CHANNEL)
    } else if (feedType == FeedType.PLAYLIST && !channelId.isNullOrBlank() && !playlistId.isNullOrBlank()) {
      videoFeedView?.setFeed(channelId, playlistId, FeedType.PLAYLIST)
    } else {
      videoFeedView?.setFeed(null, null, FeedType.DISCOVER)
    }

    // set Feed Configuration
    FWVideoPlayerUtils.setVideoFeedConfig(videoFeedView, videoFeedConfiguration)
    // set Player Configuration
    FWVideoPlayerUtils.setVideoPlayerConfig(videoPlayerConfiguration)
  }

  private fun setPlaylistGroupFeed(
    mode: String?,
    playlistGroupId: String?,
    videoFeedConfiguration: FWVideoFeedConfigModel?,
    videoPlayerConfiguration: FWVideoPlayerConfigModel?,
  ) {
    // add view
    addPlaylistGroupFeedView()

    // set layout
    val feedLayout = when {
      mode.equals(FWVideoFeedMode.Column.rawValue) -> FeedLayout.VERTICAL
      mode.equals(FWVideoFeedMode.Grid.rawValue) -> FeedLayout.GRID
      else -> FeedLayout.HORIZONTAL
    }
    playlistGroupFeedView?.setLayout(feedLayout)

    // set playlistGroupId
    playlistGroupFeedView?.setPlaylistGroupId(playlistGroupId)

    // set Feed Configuration
    FWVideoPlayerUtils.setPlaylistGroupFeedConfig(playlistGroupFeedView, videoFeedConfiguration)
    // set Player Configuration
    FWVideoPlayerUtils.setVideoPlayerConfig(videoPlayerConfiguration)
  }

  private fun addVideoFeedView() {
    removePlaylistGroupFeedView()
    if (videoFeedView != null) {
      return
    }
    videoFeedViewContainer =
      LayoutInflater.from(context).inflate(R.layout.fw_bridge_fragment_videofeed, this, false) as ConstraintLayout?
    videoFeedView = videoFeedViewContainer?.findViewById(R.id.fw_bridge_videofeed)
    if (videoFeedView != null) {
      addView(videoFeedViewContainer)
    }
  }

  private fun removeVideoFeedView() {
    if (videoFeedViewContainer == null) {
      return
    }
    removeView(videoFeedViewContainer)
    videoFeedViewContainer = null
    videoFeedView = null
  }

  private fun addPlaylistGroupFeedView() {
    removeVideoFeedView()
    if (playlistGroupFeedView != null) {
      return
    }
    playlistGroupFeedViewContainer =
      LayoutInflater.from(context).inflate(R.layout.fw_bridge_fragment_playlistfeed, this, false) as ConstraintLayout?
    playlistGroupFeedView = playlistGroupFeedViewContainer?.findViewById(R.id.fw_bridge_playlistfeed)
    if (playlistGroupFeedView != null) {
      addView(playlistGroupFeedViewContainer)
    }
  }

  private fun removePlaylistGroupFeedView() {
    if (playlistGroupFeedViewContainer == null) {
      return
    }
    removeView(playlistGroupFeedViewContainer)
    playlistGroupFeedViewContainer = null
    playlistGroupFeedView = null
  }
}
