import 'package:flutter/material.dart';

import '../../models/fw_error.dart';
import '../../models/video_feed_configuration.dart';
import '../../models/video_feed_mode.dart';
import '../../models/video_feed_source.dart';
import '../../models/video_player_configuration.dart';
import '../../utils/fw_logger_util.dart';
import '../../utils/model_utils/video_feed_configuration_util.dart';
import '../../utils/model_utils/video_feed_mode_util.dart';
import '../../utils/model_utils/video_feed_source_util.dart';
import 'video_feed_controller.dart';
import 'video_feed_platform_handler.dart';
import 'video_feed_platform_view.dart';

typedef VideoFeedCreatedCallback = void Function(
    VideoFeedController controller);
typedef VideoFeedLoadFinishedCallback = void Function(FWError? error);

/// The widget of video feed.
class VideoFeed extends StatefulWidget {
  /// Sets this to true to use hybrid composition in android side.
  /// Sets this to false to use virtual display in android side.
  /// Defaults to use virtual display.
  static bool useHybridCompositionForAndroid = false;

  /// The source of video feed.
  final VideoFeedSource source;

  /// The channel name from which content should be displayed.
  ///
  /// Videos are ordered as a channel timeline. This is required when the source is set as channel or playlist.
  final String? channel;

  /// The playlist Id for selected content.
  ///
  /// Please note channel name is necessary.
  /// Required when the source is set as playlist.
  final String? playlist;

  /// PlaylistGroup Id for selected content.
  ///
  /// Required when the source is set as playlistGroup.
  final String? playlistGroup;

  /// One of three available display modes. Defaults to row.
  final VideoFeedMode mode;

  /// The width of video feed.
  final double? width;

  /// The height of video feed.
  final double? height;

  /// The configuration of video feed.
  final VideoFeedConfiguration? videoFeedConfiguration;

  /// The configuration of video player.
  final VideoPlayerConfiguration? videoPlayerConfiguration;

  /// The callback of video feed loading result.
  ///
  /// It means loading successfully when error equals to null.
  final VideoFeedLoadFinishedCallback? onVideoFeedLoadFinished;

  /// The callback of video feed creation.
  final VideoFeedCreatedCallback? onVideoFeedCreated;

  const VideoFeed({
    Key? key,
    required this.source,
    this.channel,
    this.playlist,
    this.playlistGroup,
    this.mode = VideoFeedMode.row,
    this.width,
    this.height,
    this.videoFeedConfiguration,
    this.videoPlayerConfiguration,
    this.onVideoFeedLoadFinished,
    this.onVideoFeedCreated,
  }) : super(key: key);

  @override
  _VideoFeedState createState() => _VideoFeedState();
}

class _VideoFeedState extends State<VideoFeed> {
  VideoFeedController? _controller;

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      width: widget.width,
      height: widget.height,
      child: VideoFeedPlatformView(
        key: ValueKey(_generateKey()),
        source: widget.source,
        channel: widget.channel,
        playlist: widget.playlist,
        playlistGroup: widget.playlistGroup,
        mode: widget.mode,
        videoFeedConfiguration: widget.videoFeedConfiguration,
        videoPlayerConfiguration: widget.videoPlayerConfiguration,
        useHybridCompositionForAndroid:
            VideoFeed.useHybridCompositionForAndroid,
        onVideoFeedPlatformViewLoadFinished: widget.onVideoFeedLoadFinished,
        onVideoFeedPlatformViewCreated: _onVideoFeedPlatformViewCreated,
      ),
    );
  }

  @override
  void didUpdateWidget(covariant VideoFeed oldWidget) {
    super.didUpdateWidget(oldWidget);
    FWLoggerUtil.log("_VideoFeedState didUpdateWidget");
  }

  @override
  void dispose() {
    FWLoggerUtil.log("_VideoFeedState dispose");
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    FWLoggerUtil.log("_VideoFeedState initState");
  }

  String _generateKey() {
    final source = VideoFeedSourceUtil.convertToRawFeedSource(widget.source);
    final channel = widget.channel ?? "";
    final playlist = widget.playlist ?? "";
    final playlistGroup = widget.playlistGroup ?? "";
    final mode = VideoFeedModeUtil.convertToRawFeedMode(widget.mode);
    final titleHidden = widget.videoFeedConfiguration?.title?.hidden ?? false;
    final titlePosition = VideoFeedConfigurationUtil.convertToRawTitlePosition(
        widget.videoFeedConfiguration?.titlePosition ??
            VideoFeedTitlePosition.nested);
    final useHybridCompositionForAndroid =
        VideoFeed.useHybridCompositionForAndroid;
    String key = "source:$source";
    key += "_channel:$channel";
    key += "_playlist:$playlist";
    key += "_playlistGroup:$playlistGroup";
    key += "_mode:$mode";
    key += "_titleHidden:$titleHidden";
    key += "_titlePosition:$titlePosition";
    key += "_useHybridCompositionForAndroid:$useHybridCompositionForAndroid";

    FWLoggerUtil.log("VideoFeed key: $key");

    return key;
  }

  void _onVideoFeedPlatformViewCreated(VideoFeedPlatformHandler handler) {
    if (_controller == null) {
      _controller = VideoFeedController(handler: handler);
      widget.onVideoFeedCreated?.call(_controller!);
    } else {
      _controller!.handler = handler;
    }
  }
}
